@@ -3,13 +3,11 @@ use std::ascii::AsciiExt;
 
 use std::str::FromStr;
 
-use glib;
-use itertools::{FoldWhile, Itertools};
 use language_tags::LanguageTag;
 use locale_config::{LanguageRange, Locale};
 
-use error::*;
-use parsers::ParseError;
+use crate::error::*;
+use crate::parsers::ParseError;
 
 // No extensions at the moment.
 static IMPLEMENTED_EXTENSIONS: &[&str] = &[];
@@ -87,54 +85,30 @@ impl SystemLanguage {
     /// [`systemLanguage`]: https://www.w3.org/TR/SVG/struct.html#ConditionalProcessingSystemLanguageAttribute
     /// [BCP47]: http://www.ietf.org/rfc/bcp/bcp47.txt
     pub fn from_attribute(s: &str, locale: &Locale) -> Result<SystemLanguage, ValueErrorKind> {
-        s.split(',')
-            .map(LanguageTag::from_str)
-            .fold_while(
-                // start with no match
-                Ok(SystemLanguage(false)),
-                // The accumulator is Result<SystemLanguage, ValueErrorKind>
-                |acc, tag_result| match tag_result {
-                    Ok(language_tag) => {
-                        let have_match = acc.unwrap().0;
-                        if have_match {
-                            FoldWhile::Continue(Ok(SystemLanguage(have_match)))
-                        } else {
-                            locale_accepts_language_tag(locale, &language_tag)
-                                .map(|matches| FoldWhile::Continue(Ok(SystemLanguage(matches))))
-                                .unwrap_or_else(|e| FoldWhile::Done(Err(e)))
-                        }
+        s.split(',').map(LanguageTag::from_str).try_fold(
+            // start with no match
+            SystemLanguage(false),
+            // The accumulator is Result<SystemLanguage, ValueErrorKind>
+            |acc, tag_result| match tag_result {
+                Ok(language_tag) => {
+                    let have_match = acc.0;
+                    if have_match {
+                        Ok(SystemLanguage(have_match))
+                    } else {
+                        locale_accepts_language_tag(locale, &language_tag)
+                            .map(|matches| SystemLanguage(matches))
                     }
+                }
 
-                    Err(e) => FoldWhile::Done(Err(ValueErrorKind::Parse(ParseError::new(
-                        &format!("invalid language tag: \"{}\"", e),
-                    )))),
-                },
-            )
-            .into_inner()
+                Err(e) => Err(ValueErrorKind::Parse(ParseError::new(&format!(
+                    "invalid language tag: \"{}\"",
+                    e
+                )))),
+            },
+        )
     }
 }
 
-/// Gets the user's preferred locale from the environment and
-/// translates it to a `Locale` with `LanguageRange` fallbacks.
-///
-/// The `Locale::current()` call only contemplates a single language,
-/// but glib is smarter, and `g_get_langauge_names()` can provide
-/// fallbacks, for example, when LC_MESSAGES="en_US.UTF-8:de" (USA
-/// English and German).  This function converts the output of
-/// `g_get_language_names()` into a `Locale` with appropriate
-/// fallbacks.
-pub fn locale_from_environment() -> Locale {
-    let mut locale = Locale::invariant();
-
-    for name in glib::get_language_names() {
-        if let Ok(range) = LanguageRange::from_unix(&name) {
-            locale.add(&range);
-        }
-    }
-
-    locale
-}
-
 fn locale_accepts_language_tag(
     locale: &Locale,
     language_tag: &LanguageTag,
