@@ -10,6 +10,7 @@ use glib_sys;
 use libc;
 use markup5ever::LocalName;
 
+use crate::allowed_url::Fragment;
 use crate::parsers::ParseError;
 
 /// A simple error which refers to an attribute's value
@@ -122,6 +123,43 @@ impl From<cairo::Status> for RenderingError {
     }
 }
 
+#[derive(Debug)]
+pub enum AcquireError {
+    LinkNotFound(Fragment),
+    InvalidLinkType(Fragment),
+    CircularReference(Fragment),
+    MaxReferencesExceeded,
+}
+
+impl error::Error for AcquireError {
+    fn description(&self) -> &str {
+        match *self {
+            AcquireError::LinkNotFound(_) => "link not found",
+            AcquireError::InvalidLinkType(_) => "link is to object of invalid type",
+            AcquireError::CircularReference(_) => "circular reference in link",
+            AcquireError::MaxReferencesExceeded => "maximum number of references exceeded",
+        }
+    }
+}
+
+impl fmt::Display for AcquireError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match *self {
+            AcquireError::LinkNotFound(ref frag) =>
+                write!(f, "link not found: {}", frag),
+
+            AcquireError::InvalidLinkType(ref frag) =>
+                write!(f, "link {} is to object of invalid type", frag),
+
+            AcquireError::CircularReference(ref frag) =>
+                write!(f, "circular reference in link {}", frag),
+
+            AcquireError::MaxReferencesExceeded =>
+                write!(f, "{}", self.description()),
+        }
+    }
+}
+
 /// Helper for converting `Result<O, E>` into `Result<O, NodeError>`
 ///
 /// A `NodeError` requires a `LocalName` that corresponds to the attribute to which the
