@@ -4,7 +4,7 @@ use std::f64::consts::*;
 use crate::drawing_ctx::ViewParams;
 use crate::error::*;
 use crate::parsers::Parse;
-use crate::parsers::ParseError;
+use crate::parsers::{ParseError, finite_f32};
 use crate::properties::ComputedValues;
 
 pub type RsvgLength = Length;
@@ -234,19 +234,19 @@ impl Length {
 
             match *token {
                 Token::Number { value, .. } => Length {
-                    length: f64::from(value),
+                    length: f64::from(finite_f32(value)?),
                     unit: LengthUnit::Px,
                 },
 
                 Token::Percentage { unit_value, .. } => Length {
-                    length: f64::from(unit_value),
+                    length: f64::from(finite_f32(unit_value)?),
                     unit: LengthUnit::Percent,
                 },
 
                 Token::Dimension {
                     value, ref unit, ..
                 } => {
-                    let value = f64::from(value);
+                    let value = f64::from(finite_f32(value)?);
 
                     match unit.as_ref() {
                         "px" => Length {
