@@ -55,23 +55,13 @@ impl Parse for PaintServer {
 }
 
 pub trait PaintSource {
-    type Source;
+    type Resolved: AsPaintSource;
 
     fn resolve(
         &self,
         node: &RsvgNode,
         draw_ctx: &mut DrawingCtx,
-        bbox: &BoundingBox,
-    ) -> Result<Option<Self::Source>, RenderingError>;
-
-    fn set_pattern_on_draw_context(
-        &self,
-        pattern: &Self::Source,
-        values: &ComputedValues,
-        draw_ctx: &mut DrawingCtx,
-        opacity: &UnitInterval,
-        bbox: &BoundingBox,
-    ) -> Result<bool, RenderingError>;
+    ) -> Result<Self::Resolved, AcquireError>;
 
     fn resolve_fallbacks_and_set_pattern(
         &self,
@@ -80,25 +70,53 @@ pub trait PaintSource {
         opacity: &UnitInterval,
         bbox: &BoundingBox,
     ) -> Result<bool, RenderingError> {
-        if let Some(resolved) = self.resolve(&node, draw_ctx, bbox)? {
-            let cascaded = CascadedValues::new_from_node(node);
-            let values = cascaded.get();
-            self.set_pattern_on_draw_context(&resolved, values, draw_ctx, opacity, bbox)
-        } else {
-            Ok(false)
+        match self.resolve(&node, draw_ctx) {
+            Ok(resolved) => {
+                let cascaded = CascadedValues::new_from_node(node);
+                let values = cascaded.get();
+                resolved.set_as_paint_source(values, draw_ctx, opacity, bbox)
+            }
+
+            Err(AcquireError::CircularReference(_)) => {
+                // FIXME: add a fragment or node id to this:
+                rsvg_log!("circular reference in paint server {}", node);
+                Err(RenderingError::CircularReference)
+            }
+
+            Err(AcquireError::MaxReferencesExceeded) => {
+                rsvg_log!("maximum number of references exceeded");
+                Err(RenderingError::InstancingLimit)
+            }
+
+            Err(e) => {
+                rsvg_log!("not using paint server {}: {}", node, e);
+
+                // "could not resolve" means caller needs to fall back to color
+                Ok(false)
+            }
         }
     }
 }
 
+pub trait AsPaintSource {
+    fn set_as_paint_source(
+        self,
+        values: &ComputedValues,
+        draw_ctx: &mut DrawingCtx,
+        opacity: &UnitInterval,
+        bbox: &BoundingBox,
+    ) -> Result<bool, RenderingError>;
+}
+
 // Any of the attributes in gradient and pattern elements may be omitted.
 // The missing ones are resolved from the "fallback" IRI. If still missing,
 // they are resolved to the default value
 pub trait Resolve {
     fn is_resolved(&self) -> bool;
 
-    fn resolve_from_fallback(&mut self, fallback: &Self);
+    fn resolve_from_fallback(&self, fallback: &Self) -> Self;
 
-    fn resolve_from_defaults(&mut self);
+    fn resolve_from_defaults(&self) -> Self;
 }
 
 #[cfg(test)]
