@@ -3,9 +3,10 @@ use markup5ever::local_name;
 
 use crate::allowed_url::Fragment;
 use crate::aspect_ratio::*;
+use crate::bbox::BoundingBox;
 use crate::dpi::Dpi;
 use crate::drawing_ctx::{ClipMode, DrawingCtx, ViewParams};
-use crate::error::{AttributeResultExt, RenderingError};
+use crate::error::{AcquireError, AttributeResultExt, RenderingError};
 use crate::float_eq_cairo::ApproxEqCairo;
 use crate::length::*;
 use crate::node::*;
@@ -30,7 +31,7 @@ impl NodeTrait for NodeGroup {
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<(), RenderingError> {
+    ) -> Result<BoundingBox, RenderingError> {
         let values = cascaded.get();
 
         draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
@@ -66,7 +67,7 @@ impl NodeTrait for NodeSwitch {
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<(), RenderingError> {
+    ) -> Result<BoundingBox, RenderingError> {
         let values = cascaded.get();
 
         draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
@@ -77,7 +78,7 @@ impl NodeTrait for NodeSwitch {
             {
                 dc.draw_node_from_stack(&CascadedValues::new(cascaded, &child), &child, clipping)
             } else {
-                Ok(())
+                Ok(dc.empty_bbox())
             }
         })
     }
@@ -212,7 +213,7 @@ impl NodeTrait for NodeSvg {
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<(), RenderingError> {
+    ) -> Result<BoundingBox, RenderingError> {
         let values = cascaded.get();
 
         let params = draw_ctx.get_view_params();
@@ -305,23 +306,39 @@ impl NodeTrait for NodeUse {
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<(), RenderingError> {
+    ) -> Result<BoundingBox, RenderingError> {
         let values = cascaded.get();
 
         if self.link.is_none() {
-            return Ok(());
+            return Ok(draw_ctx.empty_bbox());
         }
 
         let link = self.link.as_ref().unwrap();
 
-        let child = if let Some(acquired) = draw_ctx.acquired_nodes().get_node(link) {
-            // Here we clone the acquired child, so that we can drop the AcquiredNode as
-            // early as possible.  This is so that the child's drawing method will be able
-            // to re-acquire the child for other purposes.
-            acquired.get().clone()
-        } else {
-            rsvg_log!("element {} references nonexistent \"{}\"", node, link,);
-            return Ok(());
+        let child = match draw_ctx.acquire_node(link, &[]) {
+            Ok(acquired) => {
+                // Here we clone the acquired child, so that we can drop the AcquiredNode as
+                // early as possible.  This is so that the child's drawing method will be able
+                // to re-acquire the child for other purposes.
+                acquired.get().clone()
+            }
+
+            Err(AcquireError::CircularReference(_)) => {
+                // FIXME: add a fragment or node id to this:
+                rsvg_log!("circular reference in <use> element {}", node);
+                return Err(RenderingError::CircularReference);
+            }
+
+            Err(AcquireError::MaxReferencesExceeded) => {
+                return Err(RenderingError::InstancingLimit);
+            }
+
+            Err(AcquireError::InvalidLinkType(_)) => unreachable!(),
+
+            Err(AcquireError::LinkNotFound(fragment)) => {
+                rsvg_log!("element {} references nonexistent \"{}\"", node, fragment);
+                return Ok(draw_ctx.empty_bbox());
+            }
         };
 
         if node.ancestors().any(|ancestor| ancestor == child) {
@@ -329,8 +346,6 @@ impl NodeTrait for NodeUse {
             return Err(RenderingError::CircularReference);
         }
 
-        draw_ctx.increase_num_elements_rendered_through_use(1);
-
         let params = draw_ctx.get_view_params();
 
         let nx = self.x.normalize(values, &params);
@@ -353,7 +368,7 @@ impl NodeTrait for NodeUse {
         // width or height set to 0 disables rendering of the element
         // https://www.w3.org/TR/SVG/struct.html#UseElementWidthAttribute
         if nw.approx_eq_cairo(0.0) || nh.approx_eq_cairo(0.0) {
-            return Ok(());
+            return Ok(draw_ctx.empty_bbox());
         }
 
         let viewport = Rectangle::new(nx, ny, nw, nh);
